<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version 1.1.2
 * ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is:  SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************
 * $Header: /advent/projects/wesat/ec_crm/sugarcrm/modules/Accounts/Accounts.php,v 1.53 2005/04/28 08:06:45 rank Exp $
 * Description:  Defines the Account SugarBean Account entity with the necessary
 * methods and variables.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 ********************************************************************************/

include_once('config.php');
require_once('include/logging.php');
require_once('include/database/PearDatabase.php');
require_once('data/SugarBean.php');
require_once('data/CRMEntity.php');
require_once('modules/Contacts/Contacts.php');
require_once('modules/Potentials/Potentials.php');
require_once('modules/Calendar/Activity.php');
require_once('modules/Notes/Notes.php');
require_once('modules/Emails/Emails.php');
require_once('include/utils/utils.php');
require_once('user_privileges/default_module_view.php');

// Account is used to store ec_account information.
class Accounts extends CRMEntity {
	var $log;
	var $db;

    var $entity_table = "ec_crmentity";
	var $tab_name = Array('ec_crmentity','ec_account','ec_accountbillads','ec_accountshipads','ec_accountscf');
	var $tab_name_index = Array('ec_crmentity'=>'crmid','ec_account'=>'accountid','ec_accountbillads'=>'accountaddressid','ec_accountshipads'=>'accountaddressid','ec_accountscf'=>'accountid');


	var $column_fields = Array();

	var $sortby_fields = Array('accountname','city','website','phone','smownerid');		


	// This is the list of ec_fields that are in the lists.
	var $list_fields = Array(
			'Account Name'=>Array('ec_account'=>'accountname'),
			'Account NO'=>Array('ec_account'=>'customernum'), 
			'Website'=>Array('ec_account'=>'website'),
			'Phone'=>Array('ec_account'=> 'phone'),
			'Assigned To'=>Array('ec_crmentity'=>'smownerid')
			);

	var $list_fields_name = Array(
			'Account Name'=>'accountname',
			'Account NO'=>'customernum',
			'Website'=>'website',
			'Phone'=>'phone',
			'Assigned To'=>'assigned_user_id'
			);
	var $list_link_field= 'accountname';

	var $search_fields = Array(
			'Account Name'=>Array('ec_account'=>'accountname'),
			'Account NO'=>Array('ec_account'=>'customernum'),
		    'Phone'=>Array('ec_account'=> 'phone'),
		    'Assigned To'=>Array('ec_crmentity'=>'smownerid')
			);

	var $search_fields_name = Array(
			'Account Name'=>'accountname',
			'Account NO'=>'customernum',
            'Phone'=>'phone',
		    'Assigned To'=>'assigned_user_id'
			);


	//Added these variables which are used as default order by and sortorder in ListView
	var $default_order_by = 'modifiedtime';
	var $default_sort_order = 'DESC';

	function Accounts() {
		$this->log =LoggerManager::getLogger('account');
		$this->db = & getSingleDBInstance();
		$this->column_fields = getColumnFields('Accounts');
	}

	/** Function to handle module specific operations when saving a entity 
	*/
	function save_module($module)
	{
		if($this->column_fields['accountname'] != "") 
		{
			$prefixa = getFirstSpell($this->column_fields['accountname']);
			$query = "update ec_account set prefix='".$prefixa."' where accountid='".$this->id."'"; 
		    $this->db->query($query);
		}
	}

    /*
	function save($module) { 
		$where_clause = "and ec_account.accountname like '%".$this->column_fields['accountname']."%'"; 
		$query = "SELECT * FROM ec_account inner join ec_crmentity on ec_crmentity.crmid=ec_account.accountid where ec_crmentity.deleted=0 $where_clause"; 
		$result =& $this->db->query($query, false, "Retrieving record $where_clause"); 
		//if($this->db->getRowCount($result ) > 1)   
		$this->db->println("account exists more than once"); 
		if ($this->db->getRowCount($result ) >= 1) {			 
			$this->db->println("account already exists");
			global $mod_strings;
			echo $mod_strings["account already exists"];
			exit();			
		}  
		else { 
			CRMEntity::save($module);
		} 
		 
	}
	*/


	// Mike Crowe Mod --------------------------------------------------------Default ordering for us
	/**
	 * Function to get sort order
 	 * return string  $sorder    - sortorder string either 'ASC' or 'DESC'
	 */
	function getSortOrder()
	{
		global $log;
                $log->debug("Entering getSortOrder() method ...");	
		if(isset($_REQUEST['sorder'])) 
			$sorder = $_REQUEST['sorder'];
		else
			$sorder = (($_SESSION['ACCOUNTS_SORT_ORDER'] != '')?($_SESSION['ACCOUNTS_SORT_ORDER']):($this->default_sort_order));
		$log->debug("Exiting getSortOrder() method ...");
		return $sorder;
	}
	/**
	 * Function to get order by
	 * return string  $order_by    - fieldname(eg: 'accountname')
 	 */
	function getOrderBy()
	{
		global $log;
                $log->debug("Entering getOrderBy() method ...");
		if (isset($_REQUEST['order_by'])) 
			$order_by = $_REQUEST['order_by'];
		else
			$order_by = (($_SESSION['ACCOUNTS_ORDER_BY'] != '')?($_SESSION['ACCOUNTS_ORDER_BY']):($this->default_order_by));
		$log->debug("Exiting getOrderBy method ...");
		return $order_by;
	}	
	// Mike Crowe Mod --------------------------------------------------------


	/** Returns a list of the associated contacts
	 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	 * All Rights Reserved..
	 * Contributor(s): ______________________________________..
	 */
	function get_contacts($id)
	{	
		global $log, $singlepane_view;
                $log->debug("Entering get_contacts(".$id.") method ...");
		global $mod_strings;

		$focus = new Contacts();

		$button = '';

		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		//SQL
		$query = "SELECT ec_contactdetails.*,
			ec_crmentity.crmid,
                        ec_crmentity.smownerid,
			case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name
			FROM ec_contactdetails
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_contactdetails.contactid
			LEFT JOIN ec_contactgrouprelation
				ON ec_contactdetails.contactid = ec_contactgrouprelation.contactid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_contactgrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_crmentity.deleted = 0
			AND ec_contactdetails.accountid = ".$id;
		$log->debug("Exiting get_contacts method ...");
		return GetRelatedList('Accounts','Contacts',$focus,$query,$button,$returnset);
	}

	/** Returns a list of the associated opportunities
	 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	 * All Rights Reserved..
	 * Contributor(s): ______________________________________..
	 */
	function get_opportunities($id)
	{
		global $log, $singlepane_view;
                $log->debug("Entering get_opportunities(".$id.") method ...");
		global $mod_strings;

		$focus = new Potentials();
		$button = '';

		
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_potential.potentialid, ec_potential.accountid,
			ec_potential.potentialname, ec_potential.sales_stage,
			ec_potential.potentialtype, ec_potential.amount,
			ec_potential.closingdate, ec_potential.potentialtype,
			case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name,
			ec_crmentity.crmid, ec_crmentity.smownerid
			FROM ec_potential
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_potential.potentialid
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			LEFT JOIN ec_potentialgrouprelation
				ON ec_potential.potentialid = ec_potentialgrouprelation.potentialid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_potentialgrouprelation.groupname
			WHERE ec_crmentity.deleted = 0
			AND ec_potential.accountid = ".$id;
		$log->debug("Exiting get_opportunities method ...");

		return GetRelatedList('Accounts','Potentials',$focus,$query,$button,$returnset);
	}

	/** Returns a list of the associated tasks
	 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	 * All Rights Reserved..
	 * Contributor(s): ______________________________________..
	 */
	function get_activities($id)
	{
		global $log, $singlepane_view;
                $log->debug("Entering get_activities(".$id.") method ...");
		global $mod_strings;

		$focus = new Activity();
		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_activity.*,
			ec_seactivityrel.*,
			ec_crmentity.crmid, ec_crmentity.smownerid,
			ec_crmentity.modifiedtime,
			case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name,
			ec_recurringevents.recurringtype
			FROM ec_activity
			INNER JOIN ec_seactivityrel
				ON ec_seactivityrel.activityid = ec_activity.activityid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_activity.activityid
			LEFT JOIN ec_users
				ON ec_users.id = ec_crmentity.smownerid
			LEFT OUTER JOIN ec_recurringevents
				ON ec_recurringevents.activityid = ec_activity.activityid
			LEFT JOIN ec_activitygrouprelation
				ON ec_activitygrouprelation.activityid = ec_crmentity.crmid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_activitygrouprelation.groupname
			WHERE ec_seactivityrel.crmid = ".$id."
			AND (activitytype='Task'
				OR activitytype='Call'
				OR activitytype='Meeting')
			AND ec_crmentity.deleted = 0
			AND ec_activity.eventstatus !=''
					AND  ec_activity.eventstatus != 'Held'";
		$log->debug("Exiting get_activities method ...");
		return GetRelatedList('Accounts','Calendar',$focus,$query,$button,$returnset);

	}
	/**
	 * Function to get Account related Task & Event which have activity type Held, Completed or Deferred.
 	 * @param  integer   $id      - accountid
 	 * returns related Task or Event record in array format
 	 */
	function get_history($id)
	{
		global $log;
                $log->debug("Entering get_history(".$id.") method ...");
		$query = "SELECT ec_activity.activityid, ec_activity.subject,
			ec_activity.status, ec_activity.eventstatus,
			ec_activity.activitytype, ec_activity.date_start, ec_activity.due_date,
			ec_crmentity.modifiedtime, ec_crmentity.createdtime,
			ec_crmentity.description,
			case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name 
			FROM ec_activity
			INNER JOIN ec_seactivityrel
				ON ec_seactivityrel.activityid = ec_activity.activityid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_activity.activityid
			LEFT JOIN ec_activitygrouprelation
				ON ec_activitygrouprelation.activityid = ec_activity.activityid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_activitygrouprelation.groupname
			INNER JOIN ec_users
				ON ec_crmentity.smcreatorid = ec_users.id
			WHERE (ec_activity.activitytype = 'Meeting'
				OR ec_activity.activitytype = 'Call'
				OR ec_activity.activitytype = 'Task')
			AND (ec_activity.status = 'Completed'
				OR ec_activity.status = 'Deferred'
				OR (ec_activity.eventstatus = 'Held'
					AND ec_activity.eventstatus != ''))
			AND ec_seactivityrel.crmid = ".$id;
		//Don't add order by, because, for security, one more condition will be added with this query in include/RelatedListView.php
		$log->debug("Exiting get_history method ...");
		return getHistory('Accounts',$query,$id);
	}
	/**
	 * Function to get Account related Attachments
 	 * @param  integer   $id      - accountid
 	 * returns related Attachment record in array format
 	 */
	function get_notes($id)
	{
		global $log,$singlepane_view;
        $log->debug("Entering get_notes(".$id.") method ...");
		require_once('modules/Notes/Notes.php');
		$focus = new Notes();
		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_notes.*,ec_account.accountname,ec_contactdetails.lastname,ec_crmentity.*,case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name
			FROM ec_notes
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_notes.notesid 
			LEFT JOIN ec_account
				ON ec_account.accountid = ec_notes.accountid 	
			LEFT JOIN ec_contactdetails
				ON ec_contactdetails.contactid = ec_notes.contact_id  
			LEFT JOIN ec_notesgrouprelation
				ON ec_notes.notesid = ec_notesgrouprelation.notesid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_notesgrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id 
			WHERE ec_account.accountid = ".$id."
			AND ec_crmentity.deleted = 0 ";
		$log->debug("Exiting get_notes method ...");
		//return getAttachmentsAndNotes('Accounts',$query,$id);
		return GetRelatedList('Accounts','Notes',$focus,$query,$button,$returnset);
	}
	/**
	* Function to get Account related Quotes
	* @param  integer   $id      - accountid
	* returns related Quotes record in array format
	*/
	function get_quotes($id)
	{
		global $log, $singlepane_view;
                $log->debug("Entering get_quotes(".$id.") method ...");
		global $app_strings;
		require_once('modules/Quotes/Quotes.php');

		$focus = new Quotes();

		$button = '';

		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name,
			ec_crmentity.*,
			ec_quotes.*,
			ec_potential.potentialname,
			ec_account.accountname
			FROM ec_quotes
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_quotes.quoteid
			LEFT OUTER JOIN ec_account
				ON ec_account.accountid = ec_quotes.accountid
			LEFT OUTER JOIN ec_potential
				ON ec_potential.potentialid = ec_quotes.potentialid
			LEFT JOIN ec_quotegrouprelation
				ON ec_quotes.quoteid = ec_quotegrouprelation.quoteid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_quotegrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_crmentity.deleted = 0
			AND ec_account.accountid = ".$id;
		$log->debug("Exiting get_quotes method ...");
		return GetRelatedList('Accounts','Quotes',$focus,$query,$button,$returnset);
	}
	/**
	* Function to get Account related Invoices 
	* @param  integer   $id      - accountid
	* returns related Invoices record in array format
	*/
	function get_invoices($id)
	{
		global $log, $singlepane_view;
                $log->debug("Entering get_invoices(".$id.") method ...");
		global $app_strings;
		require_once('modules/Invoice/Invoice.php');

		$focus = new Invoice();

		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name,
			ec_crmentity.*,
			ec_invoice.*,
			ec_account.accountname,
			ec_salesorder.subject AS salessubject
			FROM ec_invoice
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_invoice.invoiceid
			LEFT OUTER JOIN ec_account
				ON ec_account.accountid = ec_invoice.accountid
			LEFT OUTER JOIN ec_salesorder
				ON ec_salesorder.salesorderid = ec_invoice.salesorderid
			LEFT JOIN ec_invoicegrouprelation
				ON ec_invoice.invoiceid = ec_invoicegrouprelation.invoiceid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_invoicegrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_crmentity.deleted = 0
			AND ec_account.accountid = ".$id;
		$log->debug("Exiting get_invoices method ...");
		return GetRelatedList('Accounts','Invoice',$focus,$query,$button,$returnset);
	}

	/**
	* Function to get Account related SalesOrder 
	* @param  integer   $id      - accountid
	* returns related SalesOrder record in array format
	*/
	function get_salesorder($id)
	{
		global $log, $singlepane_view;
                $log->debug("Entering get_salesorder(".$id.") method ...");
		require_once('modules/SalesOrder/SalesOrder.php');
		global $app_strings;

		$focus = new SalesOrder();

		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_crmentity.*,
			ec_salesorder.*,
			ec_quotes.subject AS quotename,
			ec_account.accountname,
			case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name
			FROM ec_salesorder
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_salesorder.salesorderid
			LEFT OUTER JOIN ec_quotes
				ON ec_quotes.quoteid = ec_salesorder.quoteid
			LEFT OUTER JOIN ec_account
				ON ec_account.accountid = ec_salesorder.accountid
			LEFT JOIN ec_sogrouprelation
				ON ec_salesorder.salesorderid = ec_sogrouprelation.salesorderid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_sogrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_crmentity.deleted = 0
			AND ec_salesorder.accountid = ".$id;
		$log->debug("Exiting get_salesorder method ...");		
		return GetRelatedList('Accounts','SalesOrder',$focus,$query,$button,$returnset);
	}
	/**
	* Function to get Account related Tickets
	* @param  integer   $id      - accountid
	* returns related Ticket record in array format
	*/
	function get_tickets($id)
	{
		global $log, $singlepane_view;
                $log->debug("Entering get_tickets(".$id.") method ...");
		global $app_strings;

		$focus = new HelpDesk();
		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name, ec_users.id,
			ec_troubletickets.title, ec_troubletickets.ticketid AS crmid,
			ec_troubletickets.status, ec_troubletickets.priority,
			ec_troubletickets.parent_id,ec_ticketcf.*,
			ec_crmentity.smownerid, ec_crmentity.modifiedtime
			FROM ec_troubletickets
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_troubletickets.ticketid
			LEFT JOIN ec_ticketcf on ec_ticketcf.ticketid=ec_troubletickets.ticketid 
			LEFT JOIN ec_account
				ON ec_account.accountid = ec_troubletickets.parent_id
			LEFT JOIN ec_contactdetails
			        ON ec_contactdetails.contactid=ec_troubletickets.parent_id
			LEFT JOIN ec_users
				ON ec_users.id=ec_crmentity.smownerid
			LEFT JOIN ec_ticketgrouprelation
				ON ec_troubletickets.ticketid = ec_ticketgrouprelation.ticketid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_ticketgrouprelation.groupname
			WHERE  ec_crmentity.deleted=0 and ec_troubletickets.parent_id=".$id." or " ;

		$query .= "ec_troubletickets.parent_id in(SELECT ec_contactdetails.contactid
			FROM ec_contactdetails
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_contactdetails.contactid
			LEFT JOIN ec_contactgrouprelation
				ON ec_contactdetails.contactid = ec_contactgrouprelation.contactid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_contactgrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_crmentity.deleted = 0
			AND ec_contactdetails.accountid = ".$id;

			
		//Appending the security parameter
		global $current_user;
		require('user_privileges/user_privileges_'.$current_user->id.'.php');
        require('user_privileges/sharing_privileges_'.$current_user->id.'.php');
		$module = "Accounts";
		$tab_id=getTabid($module);
		if(($is_admin==false && $profileGlobalPermission[1] == 1 && $profileGlobalPermission[2] == 1 && $defaultOrgSharingPermission[$tab_id] == 3) || isset($_REQUEST['viewscope']) || isset($_SESSION[$module.'_viewscope']))
		{
			$sec_parameter=getListViewSecurityParameter('Contacts');
			$query .= ' '.$sec_parameter;

		}

		$query .= ") ";
		
		/*
		$query .= " UNION ALL
			SELECT ec_users.user_name, ec_users.id,
			ec_troubletickets.title, ec_troubletickets.ticketid AS crmid,
			ec_troubletickets.status, ec_troubletickets.priority,
			ec_troubletickets.parent_id,
			ec_crmentity.smownerid, ec_crmentity.modifiedtime
			FROM ec_troubletickets
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_troubletickets.ticketid
			LEFT JOIN ec_contactdetails
				ON ec_contactdetails.contactid = ec_troubletickets.parent_id
			LEFT JOIN ec_account
				ON ec_account.accountid = ec_contactdetails.accountid
			LEFT JOIN ec_users
				ON ec_users.id = ec_crmentity.smownerid
			LEFT JOIN ec_ticketgrouprelation
				ON ec_troubletickets.ticketid = ec_ticketgrouprelation.ticketid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_ticketgrouprelation.groupname
			WHERE ec_account.accountid = ".$id;
		*/	
		$log->debug("Exiting get_tickets method ...");
		return GetRelatedList('Accounts','HelpDesk',$focus,$query,$button,$returnset);
	}
	/**
	* Function to get Account related Products 
	* @param  integer   $id      - accountid
	* returns related Products record in array format
	*/
	function get_products($id)
	{
		global $log, $singlepane_view;
                $log->debug("Entering get_products(".$id.") method ...");
		require_once('modules/Products/Products.php');
		global $app_strings;

		$focus = new Products();

		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_products.productid, ec_products.productname,
			ec_products.productcode, ec_products.commissionrate,
			ec_products.qty_per_unit, ec_products.unit_price,
			ec_crmentity.crmid, ec_crmentity.smownerid
			FROM ec_products
			INNER JOIN ec_accountproductrel
				ON ec_products.productid = ec_accountproductrel.productid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_products.productid
			INNER JOIN ec_account
				ON ec_account.accountid = ec_accountproductrel.accountid
			WHERE ec_account.accountid = ".$id."
			AND ec_crmentity.deleted = 0";
		$log->debug("Exiting get_products method ...");
		return GetRelatedList('Accounts','Products',$focus,$query,$button,$returnset);
	}

	/**
	* Function to get Account related Products 
	* @param  integer   $id      - accountid
	* returns related Products record in array format
	*/
	function get_soproducts($id)
	{
		global $log, $singlepane_view;
                $log->debug("Entering get_products(".$id.") method ...");
		require_once('modules/Products/Products.php');
		global $app_strings;

		$focus = new Products();

		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_products.productid, ec_products.productname,
			ec_products.productcode, ec_products.commissionrate,
			ec_products.qty_per_unit, ec_inventoryproductrel.listprice as unit_price,
			ec_crmentity.crmid, ec_crmentity.smownerid
			FROM ec_products
			INNER JOIN ec_inventoryproductrel
				ON ec_products.productid = ec_inventoryproductrel.productid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_products.productid 
			INNER JOIN ec_salesorder on ec_salesorder.salesorderid=ec_inventoryproductrel.id 
			INNER JOIN ec_account
				ON ec_account.accountid = ec_salesorder.accountid
			WHERE ec_account.accountid = ".$id."
			AND ec_crmentity.deleted = 0";
		$log->debug("Exiting get_products method ...");
		return GetRelatedList('Accounts','Products',$focus,$query,$button,$returnset);
	}
	
		/**
	* Function to get Account related Campaigns 
	* @param  integer   $id      - accountid
	* returns related Campaigns record in array format
	*/
	function get_campaigns($id)
	{
		global $log, $singlepane_view;
                $log->debug("Entering get_products(".$id.") method ...");
		require_once('modules/Campaigns/Campaigns.php');
		global $app_strings;

		$focus = new Campaigns();

		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_campaign.*,
			ec_crmentity.crmid, ec_crmentity.smownerid,case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name, ec_users.id
			FROM ec_campaign
			INNER JOIN ec_campaignaccountrel
				ON ec_campaign.campaignid = ec_campaignaccountrel.campaignid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_campaign.campaignid
			INNER JOIN ec_account
				ON ec_account.accountid = ec_campaignaccountrel.accountid 
			LEFT JOIN ec_campaigngrouprelation
				ON ec_campaign.campaignid = ec_campaigngrouprelation.campaignid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_campaigngrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_account.accountid = ".$id."
			AND ec_crmentity.deleted = 0";
		$log->debug("Exiting get_products method ...");
		return GetRelatedList('Accounts','Campaigns',$focus,$query,$button,$returnset);
	}

	/**
	* Function to get Account related Gathers 
	* @param  integer   $id      - accountid
	* returns related Gathers record in array format
	*/
	function get_gathers($id)
	{
		global $log, $singlepane_view;
		global $app_strings;
        $log->debug("Entering get_gathers(".$id.") method ...");
		require_once('modules/Gathers/Gathers.php');
		$focus = new Gathers();
		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_gathers.*,
			ec_crmentity.crmid, ec_crmentity.smownerid,case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name, ec_users.id
			FROM ec_gathers
			INNER JOIN ec_account
				ON ec_account.accountid = ec_gathers.accountid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_gathers.gathersid 
			LEFT JOIN ec_gathersgrouprelation
				ON ec_gathers.gathersid = ec_gathersgrouprelation.gathersid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_gathersgrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_account.accountid = ".$id."
			AND ec_crmentity.deleted = 0";
		$log->debug("Exiting get_products method ...");
		return GetRelatedList('Accounts','Gathers',$focus,$query,$button,$returnset);
	}

	/** Function to export the account records in CSV Format
	* @param reference variable - order by is passed when the query is executed
	* @param reference variable - where condition is passed when the query is executed
	* Returns Export Accounts Query.
	*/
	function create_export_query(&$order_by, &$where)
	{
		global $log;
		global $current_user;
                $log->debug("Entering create_export_query(".$order_by.",".$where.") method ...");

		include("include/utils/ExportUtils.php");

		//To get the Permitted fields query and the permitted fields list
		$sql = getPermittedFieldsQuery("Accounts", "detail_view");
		global $mod_strings;
		global $current_language;
		if(empty($mod_strings)) {
			$mod_strings = return_module_language($current_language,"Accounts");
		}
		$fields_list = getFieldsListFromQuery($sql,$mod_strings);

		$query = "SELECT $fields_list FROM ".$this->entity_table."
				INNER JOIN ec_account
					ON ec_crmentity.crmid = ec_account.accountid
				LEFT JOIN ec_accountbillads
					ON ec_account.accountid = ec_accountbillads.accountaddressid
				LEFT JOIN ec_accountshipads
					ON ec_account.accountid = ec_accountshipads.accountaddressid
				LEFT JOIN ec_accountscf
					ON ec_accountscf.accountid = ec_account.accountid
				LEFT JOIN ec_accountgrouprelation
                	                ON ec_accountscf.accountid = ec_accountgrouprelation.accountid
	                        LEFT JOIN ec_groups
                        	        ON ec_groups.groupname = ec_accountgrouprelation.groupname
				LEFT JOIN ec_users
					ON ec_crmentity.smownerid = ec_users.id 
				LEFT JOIN ec_account ec_account2 
					ON ec_account2.accountid = ec_account.parentid
				";//ec_account2 is added to get the Member of account


		$where_auto = " ec_crmentity.deleted = 0 ";

		if($where != "")
			$query .= "WHERE ($where) AND ".$where_auto;
		else
			$query .= "WHERE ".$where_auto;

		require('user_privileges/user_privileges_'.$current_user->id.'.php');		
		require('user_privileges/sharing_privileges_'.$current_user->id.'.php');
		//we should add security check when the user has Private Access
		if($is_admin==false && $profileGlobalPermission[1] == 1 && $profileGlobalPermission[2] == 1 && $defaultOrgSharingPermission[6] == 3)
		{
			//Added security check to get the permitted records only
			$query = $query." ".getListViewSecurityParameter("Accounts");
		}

		if(!empty($order_by))
			$query .= " ORDER BY $order_by";

		$log->debug("Exiting create_export_query method ...");
		return $query;
	}

	/** Function to get the Columnnames of the Account Record
	* Used By vtigerCRM Word Plugin
	* Returns the Merge Fields for Word Plugin
	*/
	function getColumnNames_Acnt()
	{
		global $log,$current_user;
		$log->debug("Entering getColumnNames_Acnt() method ...");
		require('user_privileges/user_privileges_'.$current_user->id.'.php');
		if($is_admin == true || $profileGlobalPermission[1] == 0 || $profileGlobalPermission[2] == 0)
		{
			$sql1 = "SELECT fieldlabel FROM ec_field WHERE tabid = 6";
		}else
		{
			$profileList = getCurrentUserProfileList();
			$sql1 = "select fieldlabel from ec_field inner join ec_profile2field on ec_profile2field.fieldid=ec_field.fieldid inner join ec_def_org_field on ec_def_org_field.fieldid=ec_field.fieldid where ec_field.tabid=6 and ec_field.displaytype in (1,2,4) and ec_profile2field.visible=0 and ec_def_org_field.visible=0 and ec_profile2field.profileid in ".$profileList;
		} 
		$result = $this->db->query($sql1);
		$numRows = $this->db->num_rows($result);
		for($i=0; $i < $numRows;$i++)
		{
			$custom_fields[$i] = $this->db->query_result($result,$i,"fieldlabel");
			$custom_fields[$i] = ereg_replace(" ","",$custom_fields[$i]);
			$custom_fields[$i] = strtoupper($custom_fields[$i]);
		}
		$mergeflds = $custom_fields;
		$log->debug("Exiting getColumnNames_Acnt method ...");
		return $mergeflds;
	}

	function get_attachments($id)
	{
		global $log;
		$log->debug("Entering get_attachments(".$id.") method ...");
		$query = "SELECT ec_attachments.description AS title,
				'Attachments' AS ActivityType,
			ec_attachments.name AS filename,
			ec_attachments.type AS FileType,
				crm2.modifiedtime AS lastmodified,
			ec_attachments.attachmentsid AS attachmentsid,
			ec_seattachmentsrel.attachmentsid AS crmid,
			crm2.createdtime,
			ec_attachments.description, ec_users.user_name
			FROM ec_attachments
			INNER JOIN ec_seattachmentsrel
				ON ec_seattachmentsrel.attachmentsid = ec_attachments.attachmentsid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_seattachmentsrel.crmid
			INNER JOIN ec_crmentity AS crm2
				ON crm2.crmid = ec_attachments.attachmentsid
			INNER JOIN ec_users
				ON ec_crmentity.smcreatorid = ec_users.id
			WHERE ec_crmentity.crmid = ".$id;			

		$log->debug("Exiting get_attachments method ...");
		return getAttachments('Accounts',$query,$id);
	}

	/**
	* Function to get PurchaseOrder related Tuihuos 
	* @param  integer   $id      - purchaseorderid
	* returns related Tuihuos record in array format
	*/
	function get_tuihuos($id)
	{
		global $log, $singlepane_view;
		global $app_strings;
        $log->debug("Entering get_tuihuos(".$id.") method ...");
		require_once('modules/Tuihuos/Tuihuos.php');
		$focus = new Tuihuos();
		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_tuihuos.*,
			ec_crmentity.crmid, ec_crmentity.smownerid,case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name
			FROM ec_tuihuos
			INNER JOIN ec_account
				ON ec_account.accountid = ec_tuihuos.accountid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_tuihuos.tuihuosid 
			LEFT JOIN ec_tuihuosgrouprelation
				ON ec_tuihuos.tuihuosid = ec_tuihuosgrouprelation.tuihuosid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_tuihuosgrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_tuihuos.accountid = ".$id."
			AND ec_crmentity.deleted = 0";
		$log->debug("Exiting get_tuihuos method ...");
		return GetRelatedList('Accounts','Tuihuos',$focus,$query,$button,$returnset);
	}

	function get_cares($id)
	{
		global $log, $singlepane_view;
		global $app_strings;
        $log->debug("Entering get_cares(".$id.") method ...");
		require_once('modules/Cares/Cares.php');
		$focus = new Cares();
		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_cares.*,
			ec_crmentity.crmid, ec_crmentity.smownerid,case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name
			FROM ec_cares
			INNER JOIN ec_account
				ON ec_account.accountid = ec_cares.accountid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_cares.caresid 
			LEFT JOIN ec_caresgrouprelation
				ON ec_cares.caresid = ec_caresgrouprelation.caresid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_caresgrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_cares.accountid = ".$id."
			AND ec_crmentity.deleted = 0";
		$log->debug("Exiting get_cares method ...");
		return GetRelatedList('Accounts','Cares',$focus,$query,$button,$returnset);
	}
	function get_projects($id)
	{
		global $log, $singlepane_view;
		global $app_strings;
        $log->debug("Entering get_projects(".$id.") method ...");
		require_once('modules/Projects/Projects.php');
		$focus = new Projects();
		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;

		$query = "SELECT ec_projects.*,
			ec_crmentity.crmid, ec_crmentity.smownerid,case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name
			FROM ec_projects
			INNER JOIN ec_account
				ON ec_account.accountid = ec_projects.accountid
			INNER JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_projects.projectsid 
			LEFT JOIN ec_projectsgrouprelation
				ON ec_projects.projectsid = ec_projectsgrouprelation.projectsid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_projectsgrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_projects.accountid = ".$id."
			AND ec_crmentity.deleted = 0";
		$log->debug("Exiting get_projects method ...");
		return GetRelatedList('Accounts','Projects',$focus,$query,$button,$returnset);
	}

	function get_assignhistory($id)
	{	
		global $log;
		$log->debug("Entering get_assignhistory(".$id.") method ...");

		global $adb;
		global $mod_strings;
		global $app_strings;

		$query = 'select ec_assignhistory.*,ec_users.user_name,ec_users.id from ec_assignhistory inner join ec_users on ec_users.id=ec_assignhistory.userid where accountid = '.$id.' order by assigntime desc';
		$result=$adb->query($query);
		$noofrows = $adb->num_rows($result);

		$header[] = $mod_strings['Assigned To'];
		$header[] = $mod_strings['startdate'];
		$header[] = $mod_strings['enddate'];
		$header[] = $mod_strings['assigntime'];

		while($row = $adb->fetch_array($result))
		{
			$entries = Array();

			$entries[] = $row['user_name'];
			$entries[] = $row['startdate'];
			$entries[] = $row['enddate'];
			$entries[] = getDisplayDate($row['assigntime']);

			$entries_list[] = $entries;
		}

		$return_data = Array('header'=>$header,'entries'=>$entries_list);

	 	$log->debug("Exiting get_assignhistory method ...");

		return $return_data;
	}
	/**
	* Function to get related custom module records
	* @param  integer   $id  - current tab record id
	* returns related custom module records in array format
	*/
	function get_generalmodules($id,$related_tabname)
	{
		global $log, $singlepane_view;
        $log->debug("Entering get_generalmodules() method ...");
		require_once("modules/$related_tabname/$related_tabname.php");
		global $app_strings;
		$focus = new $related_tabname();
		$button = '';
		if($singlepane_view == 'true')
			$returnset = '&return_module=Accounts&return_action=DetailView&return_id='.$id;
		else
			$returnset = '&return_module=Accounts&return_action=CallRelatedList&return_id='.$id;
		$related_bean = substr($related_tabname,0,-1);
		$related_bean = strtolower($related_bean);
		$query = "SELECT ec_".$related_bean."s.*,
			ec_crmentity.crmid, ec_crmentity.smownerid,case when (ec_users.user_name not like '') then ec_users.user_name else ec_groups.groupname end as user_name
			FROM ec_".$related_bean."s
			INNER JOIN ec_account
				ON ec_account.accountid = ec_".$related_bean."s.accountid
			INNER JOIN ec_".$related_bean."scf 
				ON ec_".$related_bean."scf.".$related_bean."sid = ec_".$related_bean."s.".$related_bean."sid 
			LEFT JOIN ec_crmentity
				ON ec_crmentity.crmid = ec_".$related_bean."s.".$related_bean."sid 
			LEFT JOIN ec_".$related_bean."sgrouprelation
				ON ec_".$related_bean."s.".$related_bean."sid = ec_".$related_bean."sgrouprelation.".$related_bean."sid
			LEFT JOIN ec_groups
				ON ec_groups.groupname = ec_".$related_bean."sgrouprelation.groupname
			LEFT JOIN ec_users
				ON ec_crmentity.smownerid = ec_users.id
			WHERE ec_".$related_bean."s.accountid = ".$id."
			AND ec_crmentity.deleted = 0";
		$log->debug("Exiting get_generalmodules method ...");
		return GetRelatedList("Accounts",$related_tabname,$focus,$query,$button,$returnset);
	}

	function get_next_id() {
		//$query = "select count(*) as num from ec_account";
		//$result = $this->db->query($query);		
		//$num = $this->db->query_result($result,0,'num') + 1;
		$num = $this->db->getUniqueID("ec_account");
		if($num > 99) return $num;
		elseif($num > 9) return "0".$num;
		else return "00".$num;
	}

}

?>
